from dataclasses import asdict
import json
import time
from typing import Optional, Union, List
from urllib.parse import urljoin

import pandas as pd
import requests
from colorama import init as colorama_init
from termcolor import colored

from metlo.config import get_config
from metlo.types.query import Filter, TimeDimension
from metlo.utils import DateTimeEncoder
from metlo.load_definitions import load_defs


def query(
    metrics: Union[str, List[str]],
    filters: List[Filter] = [],
    groups: List[str] = [],
    time_dimensions: List[TimeDimension] = [],
    streaming = False,
) -> Optional[pd.DataFrame]:
    colorama_init()
    if not isinstance(metrics, list):
        metrics = [metrics]

    conf = get_config()
    if not conf:
        return

    query_data = {
        'metrics': metrics,
        'filters': [asdict(e) for e in filters],
        'groups': groups,
        'time_dimensions': [asdict(e) for e in time_dimensions],
        'streaming': streaming,
    }

    if conf.definition_dir:
        request_data = {
            'query': query_data,
            'definitions': [
                asdict(e) for e in load_defs(conf.definition_dir)
            ]
        }
    else:
        request_data = query_data

    query_url = urljoin(conf.host_name, 'api/query')
    res = requests.post(
        query_url,
        data=json.dumps(request_data, cls=DateTimeEncoder),
        headers={
            'Content-type': 'application/json',
            'Authorization': f'Bearer {conf.api_key}',
        },
    )
    if not res.ok:
        print(colored(f'Query Failed: {res.status_code}', 'red'))
        return
    query_res = res.json()

    if not query_res['ok']:
        print(query_res)
        return

    if query_res.get('result'):
        return pd.DataFrame(query_res['result'])
    
    poll_id = query_res['id']
    poll_url = urljoin(conf.host_name, f'api/fetch/{poll_id}')
    fetch_status = 'PENDING'
    poll_res = None

    while fetch_status in {'PENDING', 'RECEIVED', 'STARTED', 'RETRY'}:
        poll_res = requests.get(
            poll_url,
            headers={ 'Authorization': f'Bearer {conf.api_key}' },
        ).json()
        fetch_status = poll_res['status']
        time.sleep(1)
    
    if fetch_status == 'FAILURE':
        print(colored(f'Query Failed', 'red'))

    if poll_res.get('result'):
        return pd.DataFrame(poll_res['result'])
    if poll_res.get('status') == 'SUCCESS':
        return pd.DataFrame()

    print(poll_res)



from typing import Optional
from datetime import datetime
from pydantic import conlist
from pydantic.dataclasses import dataclass

from metlo.types.enums import FilterOp, TimeGranularity


@dataclass
class Filter:
    dimension: str
    values: list
    op: FilterOp = FilterOp.EQ


@dataclass
class TimeDimension:
    dimension: str
    granularity: TimeGranularity
    date_range: Optional[conlist(datetime, min_items=2, max_items=2)] = None



import argparse
from getpass import getpass
from glob import glob
import os

from colorama import init as colorama_init
from termcolor import colored
from pydantic import ValidationError

from metlo.config import (
    DEFAULT_CONFIG_PATH, DEFAULT_CONFIG_FOLDER, API_KEY_NAME, HOST_KEY_NAME
)
from metlo.load_definitions import load_single_def


def setup():
    host = input('Enter your Metlo Host: ')
    api_key = getpass(prompt='Enter your API Key: ')

    if not os.path.exists(DEFAULT_CONFIG_FOLDER):
        os.mkdir(DEFAULT_CONFIG_FOLDER)

    with open(DEFAULT_CONFIG_PATH, 'w') as f:
        f.write(f'{HOST_KEY_NAME}={host}\n{API_KEY_NAME}={api_key}')


def validate(directory: str):
    yaml_paths = glob(os.path.join(directory, '*.yaml'))
    for yaml_path in yaml_paths:
        try:
            print(f'Validating {yaml_path}')
            load_single_def(yaml_path)
            print(colored('PASSED', 'green'))
        except ValidationError as e:
            print(colored(str(e), 'red'))

def main():
    colorama_init()
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest='command', required=True)
    setup_parser = subparsers.add_parser('setup')
    validate_parser = subparsers.add_parser('validate')
    validate_parser.add_argument(
        '-d', '--directory', help='The definition directory to validate.', required=True
    )
    args = parser.parse_args()

    command = args.command 

    if command == 'setup':
        setup()
    if command == 'validate':
        validate(args.directory)


if __name__ == '__main__':
    main()



import setuptools
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / 'README.md').read_text()

with open(this_directory / 'requirements.in') as f:
    required = f.read().strip().splitlines()

setuptools.setup(
    name='metlo',
    version='0.0.8',
    author='S2 Labs Inc.',
    author_email='akshay@metlo.com',
    description='Metlo\'s Python SDK',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=setuptools.find_packages(),
    python_requires='>=3.0',
    license="MIT",
    entry_points = {
        'console_scripts': ['metlo=metlo.command_line:main'],
    },
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    project_urls={
        'Homepage': 'https://www.metlo.com',
        'Documentation': 'https://docs.metlo.com',
        'Source Code': 'https://github.com/metlo-labs/metlo-python',
    },
    py_modules=['metlo'],
    package_dir={'':'.'},
    install_requires=required,
)
